from django.urls import reverse
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import render
from django.apps import apps

from municipios.forms import FormMunicipio


def base_url_js(request):
    return HttpResponse(u"var __municipios_base_url__ = '%s';" % reverse('municipios-base-url'))


def municipios_ajax(request, uf, app_label, object_name):
    model_cls = apps.get_model(app_label, object_name)

    municipio_list = model_cls.objects.filter(Q(uf=uf)).order_by('nome')
    return render(
        request,
        "municipios/municipios_options.html",
        {"municipio_list": municipio_list},
    )


def teste(request):
    form = FormMunicipio(request.GET or None)
    return render(
        request,
        'municipios/teste.html',
        {'form': form},
    )



# coding: utf-8
from django.db import models
from django.forms.models import ModelChoiceField
from widgets import SelectMunicipioWidget
from models import Municipio


class MunicipioChoiceField(ModelChoiceField):
    def __init__(self, *args, **kwargs):
        super(MunicipioChoiceField, self).__init__(*args, **kwargs)

        app_label = self.queryset.model._meta.app_label
        object_name = self.queryset.model._meta.object_name

        self.widget = SelectMunicipioWidget(app_label=app_label, object_name=object_name)

        extra_attrs = self.widget_attrs(self.widget)
        if extra_attrs:
            self.widget.attrs.update(extra_attrs)


class MunicipioForeignKey(models.ForeignKey):

    description = "Chave Estrangeira para um Municipio"

    def __init__(self, to=None, *args, **kwargs):
        if not to:
            to = Municipio
        super(MunicipioForeignKey, self).__init__(to, *args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'form_class': MunicipioChoiceField}
        defaults.update(kwargs)
        return super(MunicipioForeignKey, self).formfield(**defaults)



try:
    from django.conf.urls import url
except:
    from django.conf.urls.defaults import url

import municipios.views as mviews

urlpatterns = (
    url(r'^$', mviews.base_url_js, name='municipios-base-url'),
    url(r'^base_url.js$', mviews.base_url_js, name='municipios-base-url-js'),
    url(r'^ajax/municipios/(?P<uf>\w\w)/(?P<app_label>\w+)/(?P<object_name>\w+)/$', mviews.municipios_ajax, name='municipios-ajax'),
    url(r'^teste/', mviews.teste, name='municipios-teste'),
)



# -*- coding: utf-8 -*-
from django.forms.widgets import Select, Widget

from django.template.loader import get_template

from django.utils.safestring import mark_safe

from django.apps import apps

try:
    from django.core.urlresolvers import reverse_lazy
    HAS_REVERSE_LAZY = True
except ImportError:
    HAS_REVERSE_LAZY = False


class SelectMunicipioWidget(Widget):
    """
    Widget to render a <select> with the UFs (Brazillian states)
    and a <select> with the actual Municipio depending on the state
    selected
    """
    def __init__(self, *args, **kwargs):
        self.app_label = kwargs.pop('app_label', 'municipios')
        self.object_name = kwargs.pop('object_name', 'Municipio')
        self.mun_cls = apps.get_model(self.app_label, self.object_name)
        super(SelectMunicipioWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        attrs = attrs or {}
        if attrs:
            self.attrs.update(attrs)

        output = []
        uf_val = ''
        uf_choices = [('', '--')] + list(self.mun_cls.objects.values_list('uf__id_ibge', 'uf_sigla').distinct().order_by('uf_sigla'))
        uf_select = Select(choices=uf_choices)
        municipio_select = Select(choices=(('', '--'),))

        if value:
            try:
                municipio = self.mun_cls.objects.get(pk=value)
                uf_val = municipio.uf.pk
                mun_choices = [(m.pk, m.nome) for m in self.mun_cls.objects.filter(uf=uf_val).order_by('nome')]
                municipio_select = Select(choices=[('', '- Selecione -')] + mun_choices)
            except self.mun_cls.DoesNotExist:
                pass
        uf_attrs = self.attrs.copy()
        uf_attrs.update(
            {
                "id": '%s_uf' % name,
                "onchange": "changeUF(this);",
                "data-app_label": self.app_label,
                "data-object_name": self.object_name,
            }
        )
        select_html = uf_select.render('%s_uf' % name, uf_val, attrs=uf_attrs)
        required = False
        if 'class' in self.attrs:
            required = self.attrs['class']
        # output.append(u'<div class="field"><label%s>UF</label><br />%s</div>' % (required and ' class="required"' or '', select_html))
        template_uf = get_template('municipios/uf_field.html')
        context_uf = {'label_class': required, 'wselect': select_html}
        output.append(template_uf.render(context_uf))

        munic_attrs = self.attrs.copy()
        munic_attrs['style'] = "width:250px;"
        select_html = municipio_select.render(name, value, munic_attrs)
        # output.append(u'<div class="field"><label%s>Município</label><br />%s</div>' % (required and ' class="required"' or '', select_html))
        template_mun = get_template('municipios/municipio_field.html')
        context_mun = {'label_class': required, 'wselect': select_html}
        output.append(template_mun.render(context_mun))

        return mark_safe(u'\n'.join(output))

    class Media:
        if HAS_REVERSE_LAZY:
            base_url_js = reverse_lazy('municipios-base-url-js')
        else:
            base_url_js = '/municipios_app/base_url.js'
        js = (base_url_js, 'municipios/js/municipio.js',)
